==> ui-18.0.1.tgz: package/components/_selectbox/Selectbox.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { useEffect, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import { fromEvent } from 'rxjs';
import { useForkRef, useRefExtra, useResize } from '@react-devui/hooks';
import { CloseCircleFilled, DownOutlined, LoadingOutlined, SearchOutlined } from '@react-devui/icons';
import { checkNodeExist, getClassName } from '@react-devui/utils';
import { useMaxIndex } from '../../hooks';
import { cloneHTMLElement } from '../../utils';
import { DBaseDesign } from '../_base-design';
import { DBaseInput } from '../_base-input';
import { DFocusVisible } from '../_focus-visible';
import { useGlobalScroll, useLayout, usePrefixConfig, useTranslation } from '../root';
export function DSelectbox(props) {
    const { children, dRef, dClassNamePrefix, dFormControl, dVisible, dContent, dContentTitle, dPlaceholder, dSuffix, dSize, dLoading, dSearchable, dClearable, dDisabled, dUpdatePosition, dInputRender, onFocusVisibleChange, onClear } = props, restProps = __rest(props, ["children", "dRef", "dClassNamePrefix", "dFormControl", "dVisible", "dContent", "dContentTitle", "dPlaceholder", "dSuffix", "dSize", "dLoading", "dSearchable", "dClearable", "dDisabled", "dUpdatePosition", "dInputRender", "onFocusVisibleChange", "onClear"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { dPageScrollRef, dContentResizeRef } = useLayout();
    //#endregion
    //#region Ref
    const boxRef = useRef(null);
    const inputRef = useRef(null);
    const containerRef = useRefExtra(() => {
        let el = document.getElementById(`${prefix}-root`);
        if (!el) {
            el = document.createElement('div');
            el.id = `${prefix}-root`;
            document.body.appendChild(el);
        }
        return el;
    }, true);
    const combineBoxRef = useForkRef(boxRef, dRef.box);
    const combineInputRef = useForkRef(inputRef, dRef.input);
    //#endregion
    const prefix = `${dPrefix}${dClassNamePrefix}`;
    const [t] = useTranslation();
    const [isFocus, setIsFocus] = useState(false);
    const inputable = dSearchable && dVisible;
    const clearable = dClearable && dContent && !dVisible && !dLoading && !dDisabled;
    const maxZIndex = useMaxIndex(dVisible);
    const preventBlur = (e) => {
        if (e.target !== inputRef.current && e.button === 0) {
            e.preventDefault();
        }
    };
    const globalScroll = useGlobalScroll(dUpdatePosition.fn, !dVisible);
    useEffect(() => {
        if (!dVisible || !globalScroll) {
            const ob = fromEvent([dPageScrollRef, ...dUpdatePosition.extraScrollRefs].map((ref) => ref === null || ref === void 0 ? void 0 : ref.current).filter((el) => el), 'scroll', { passive: true }).subscribe({
                next: () => {
                    dUpdatePosition.fn();
                },
            });
            return () => {
                ob.unsubscribe();
            };
        }
    });
    useResize(boxRef, dUpdatePosition.fn, !dVisible);
    useResize(dUpdatePosition.popupRef, dUpdatePosition.fn, !dVisible);
    useResize(dContentResizeRef, dUpdatePosition.fn, !dVisible);
    return (_jsxs(_Fragment, { children: [_jsx(DBaseDesign, Object.assign({ dComposeDesign: {
                    active: isFocus,
                    disabled: dDisabled,
                }, dFormDesign: {
                    control: dFormControl,
                } }, { children: ({ render: renderBaseDesign }) => renderBaseDesign(_jsxs("div", Object.assign({}, restProps, { ref: combineBoxRef, className: getClassName(restProps.className, prefix, {
                        [`${prefix}--${dSize}`]: dSize,
                        'is-expanded': dVisible,
                        'is-focus': isFocus,
                        'is-disabled': dDisabled,
                    }), onMouseDown: (e) => {
                        var _a;
                        (_a = restProps.onMouseDown) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                        preventBlur(e);
                    }, onMouseUp: (e) => {
                        var _a;
                        (_a = restProps.onMouseUp) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                        preventBlur(e);
                    }, onClick: (e) => {
                        var _a, _b;
                        (_a = restProps.onClick) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                        (_b = inputRef.current) === null || _b === void 0 ? void 0 : _b.focus({ preventScroll: true });
                    } }, { children: [_jsxs("div", Object.assign({ className: `${prefix}__container`, title: dContentTitle }, { children: [_jsx(DFocusVisible, Object.assign({ onFocusVisibleChange: onFocusVisibleChange }, { children: ({ render: renderFocusVisible }) => (_jsx(DBaseInput, Object.assign({ dFormControl: dFormControl, dLabelFor: true }, { children: ({ render: renderBaseInput }) => {
                                            return dInputRender(renderFocusVisible(renderBaseInput(_jsx("input", { ref: combineInputRef, className: `${prefix}__search`, style: {
                                                    opacity: inputable ? undefined : 0,
                                                    zIndex: inputable ? undefined : -1,
                                                }, type: "text", autoComplete: "off", disabled: dDisabled, role: "combobox", "aria-haspopup": "listbox", "aria-expanded": dVisible, onFocus: () => {
                                                    setIsFocus(true);
                                                }, onBlur: () => {
                                                    setIsFocus(false);
                                                } }))));
                                        } }))) })), !inputable &&
                                    (dContent ? (_jsx("div", Object.assign({ className: `${prefix}__content` }, { children: dContent }))) : dPlaceholder ? (_jsx("div", Object.assign({ className: `${prefix}__placeholder-wrapper` }, { children: _jsx("div", Object.assign({ className: `${prefix}__placeholder` }, { children: dPlaceholder })) }))) : null)] })), checkNodeExist(dSuffix) && (_jsx("div", Object.assign({ className: `${prefix}__suffix`, onClick: (e) => {
                                e.stopPropagation();
                            } }, { children: dSuffix }))), clearable && (_jsx("div", Object.assign({ className: `${prefix}__clear`, role: "button", "aria-label": t('Clear'), onClick: (e) => {
                                e.stopPropagation();
                                onClear();
                            } }, { children: _jsx(CloseCircleFilled, {}) }))), _jsx("div", Object.assign({ className: getClassName(`${prefix}__icon`, {
                                'is-arrow-up': !dLoading && !inputable && dVisible,
                            }), style: {
                                opacity: clearable ? 0 : 1,
                            } }, { children: dLoading ? _jsx(LoadingOutlined, { dSpin: true }) : inputable ? _jsx(SearchOutlined, {}) : _jsx(DownOutlined, {}) }))] }))) })), containerRef.current &&
                ReactDOM.createPortal(children({
                    renderPopup: (el) => cloneHTMLElement(el, {
                        style: Object.assign(Object.assign({}, el.props.style), { zIndex: maxZIndex }),
                        onMouseDown: (e) => {
                            var _a, _b;
                            (_b = (_a = el.props).onMouseDown) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                            preventBlur(e);
                        },
                        onMouseUp: (e) => {
                            var _a, _b;
                            (_b = (_a = el.props).onMouseUp) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                            preventBlur(e);
                        },
                    }),
                }), containerRef.current)] }));
}

==> ui-18.1.0.tgz: package/components/_selectbox/Selectbox.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { useEffect, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import { fromEvent } from 'rxjs';
import { useForkRef, useRefExtra, useResize } from '@react-devui/hooks';
import { CloseCircleFilled, DownOutlined, LoadingOutlined, SearchOutlined } from '@react-devui/icons';
import { checkNodeExist, getClassName } from '@react-devui/utils';
import { useMaxIndex } from '../../hooks';
import { cloneHTMLElement } from '../../utils';
import { EXPANDED_DATA } from '../../utils/checkNoExpandedEl';
import { DBaseDesign } from '../_base-design';
import { DBaseInput } from '../_base-input';
import { DFocusVisible } from '../_focus-visible';
import { useGlobalScroll, useLayout, usePrefixConfig, useTranslation } from '../root';
export function DSelectbox(props) {
    const { children, dRef, dClassNamePrefix, dFormControl, dVisible, dContent, dContentTitle, dPlaceholder, dSuffix, dSize, dLoading, dSearchable, dClearable, dDisabled, dUpdatePosition, dInputRender, onFocusVisibleChange, onClear } = props, restProps = __rest(props, ["children", "dRef", "dClassNamePrefix", "dFormControl", "dVisible", "dContent", "dContentTitle", "dPlaceholder", "dSuffix", "dSize", "dLoading", "dSearchable", "dClearable", "dDisabled", "dUpdatePosition", "dInputRender", "onFocusVisibleChange", "onClear"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { dPageScrollRef, dContentResizeRef } = useLayout();
    //#endregion
    //#region Ref
    const boxRef = useRef(null);
    const inputRef = useRef(null);
    const containerRef = useRefExtra(() => {
        let el = document.getElementById(`${prefix}-root`);
        if (!el) {
            el = document.createElement('div');
            el.id = `${prefix}-root`;
            document.body.appendChild(el);
        }
        return el;
    }, true);
    const combineBoxRef = useForkRef(boxRef, dRef.box);
    const combineInputRef = useForkRef(inputRef, dRef.input);
    //#endregion
    const prefix = `${dPrefix}${dClassNamePrefix}`;
    const [t] = useTranslation();
    const [isFocus, setIsFocus] = useState(false);
    const inputable = dSearchable && dVisible;
    const clearable = dClearable && dContent && !dVisible && !dLoading && !dDisabled;
    const maxZIndex = useMaxIndex(dVisible);
    const preventBlur = (e) => {
        if (e.target !== inputRef.current && e.button === 0) {
            e.preventDefault();
        }
    };
    const globalScroll = useGlobalScroll(dUpdatePosition.fn, !dVisible);
    useEffect(() => {
        if (!dVisible || !globalScroll) {
            const ob = fromEvent([dPageScrollRef, ...dUpdatePosition.extraScrollRefs].map((ref) => ref === null || ref === void 0 ? void 0 : ref.current).filter((el) => el), 'scroll', { passive: true }).subscribe({
                next: () => {
                    dUpdatePosition.fn();
                },
            });
            return () => {
                ob.unsubscribe();
            };
        }
    });
    useResize(boxRef, dUpdatePosition.fn, !dVisible);
    useResize(dUpdatePosition.popupRef, dUpdatePosition.fn, !dVisible);
    useResize(dContentResizeRef, dUpdatePosition.fn, !dVisible);
    return (_jsxs(_Fragment, { children: [_jsx(DBaseDesign, Object.assign({ dComposeDesign: {
                    active: isFocus,
                    disabled: dDisabled,
                }, dFormDesign: {
                    control: dFormControl,
                } }, { children: ({ render: renderBaseDesign }) => renderBaseDesign(_jsxs("div", Object.assign({}, restProps, { [EXPANDED_DATA]: dVisible }, { ref: combineBoxRef, className: getClassName(restProps.className, prefix, {
                        [`${prefix}--${dSize}`]: dSize,
                        'is-expanded': dVisible,
                        'is-focus': isFocus,
                        'is-disabled': dDisabled,
                    }), onMouseDown: (e) => {
                        var _a;
                        (_a = restProps.onMouseDown) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                        preventBlur(e);
                    }, onMouseUp: (e) => {
                        var _a;
                        (_a = restProps.onMouseUp) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                        preventBlur(e);
                    }, onClick: (e) => {
                        var _a, _b;
                        (_a = restProps.onClick) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                        (_b = inputRef.current) === null || _b === void 0 ? void 0 : _b.focus({ preventScroll: true });
                    } }, { children: [_jsxs("div", Object.assign({ className: `${prefix}__container`, title: dContentTitle }, { children: [_jsx(DFocusVisible, Object.assign({ onFocusVisibleChange: onFocusVisibleChange }, { children: ({ render: renderFocusVisible }) => (_jsx(DBaseInput, Object.assign({ dFormControl: dFormControl, dLabelFor: true }, { children: ({ render: renderBaseInput }) => {
                                            return dInputRender(renderFocusVisible(renderBaseInput(_jsx("input", { ref: combineInputRef, className: `${prefix}__search`, style: {
                                                    opacity: inputable ? undefined : 0,
                                                    zIndex: inputable ? undefined : -1,
                                                }, type: "text", autoComplete: "off", disabled: dDisabled, role: "combobox", "aria-haspopup": "listbox", "aria-expanded": dVisible, onFocus: () => {
                                                    setIsFocus(true);
                                                }, onBlur: () => {
                                                    setIsFocus(false);
                                                } }))));
                                        } }))) })), !inputable &&
                                    (dContent ? (_jsx("div", Object.assign({ className: `${prefix}__content` }, { children: dContent }))) : dPlaceholder ? (_jsx("div", Object.assign({ className: `${prefix}__placeholder-wrapper` }, { children: _jsx("div", Object.assign({ className: `${prefix}__placeholder` }, { children: dPlaceholder })) }))) : null)] })), checkNodeExist(dSuffix) && (_jsx("div", Object.assign({ className: `${prefix}__suffix`, onClick: (e) => {
                                e.stopPropagation();
                            } }, { children: dSuffix }))), clearable && (_jsx("div", Object.assign({ className: `${prefix}__clear`, role: "button", "aria-label": t('Clear'), onClick: (e) => {
                                e.stopPropagation();
                                onClear();
                            } }, { children: _jsx(CloseCircleFilled, {}) }))), _jsx("div", Object.assign({ className: getClassName(`${prefix}__icon`, {
                                'is-arrow-up': !dLoading && !inputable && dVisible,
                            }), style: {
                                opacity: clearable ? 0 : 1,
                            } }, { children: dLoading ? _jsx(LoadingOutlined, { dSpin: true }) : inputable ? _jsx(SearchOutlined, {}) : _jsx(DownOutlined, {}) }))] }))) })), containerRef.current &&
                ReactDOM.createPortal(children({
                    renderPopup: (el) => cloneHTMLElement(el, {
                        style: Object.assign(Object.assign({}, el.props.style), { zIndex: maxZIndex }),
                        onMouseDown: (e) => {
                            var _a, _b;
                            (_b = (_a = el.props).onMouseDown) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                            preventBlur(e);
                        },
                        onMouseUp: (e) => {
                            var _a, _b;
                            (_b = (_a = el.props).onMouseUp) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                            preventBlur(e);
                        },
                    }),
                }), containerRef.current)] }));
}
